# -*- coding: utf-8 -*-

#changement de dossier
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/gradient_descent")

#importation des données
import pandas
D = pandas.read_table("sonar.txt",delimiter="\t",header=0)

#dimensions
print(D.shape)

#liste des variables
print(D.columns)

#convertir
D = D.as_matrix()

#variables prédictives jusqu'à l'avant-dernière
X = D[:,:-1]
print(X.shape)

#variable cible - la dernière
y = D[:,-1]
print(y.shape)

#numpy
import numpy

#décompte des effectifs des classes
print(numpy.unique(y,return_counts=True))

#version de scikit-learn !!! important pour les options
import sklearn
print(sklearn.__version__)

#initialisation
from sklearn.linear_model import SGDClassifier
msgd1 = SGDClassifier(loss="log",penalty="none",learning_rate="constant",eta0=0.1, max_iter=100,tol=None)

#modélisation
msgd1.fit(X,y)

#nombre d'itération
print(msgd1.n_iter_)

#probabilités conditionnelles
probas = msgd1.predict_proba(X)
print(probas[:10,:])

#valeur de la fonction de perte
from sklearn import metrics
logloss = metrics.log_loss(y,probas[:,1],normalize=True,eps=1e-15)
print(logloss)

#seuils pour probas
eps = 1e-15

#probabilités conditionnelles P(Y=1/X)
p = probas[:,1]

#corriger les probas
p[p<eps] = eps
p[p>(1-eps)] = 1-eps

#calcul manuel
import numpy
print(numpy.mean(-(y*numpy.log(p)+(1-y)*numpy.log(1-p))))

#AUC en validation croisée
from sklearn import model_selection

#préparer la constitution en groupes - pour avoir les mêmes à chaque lancement
cvEval = model_selection.StratifiedKFold(n_splits=3,shuffle=True,random_state=1)

#AUC en validation croisée
auc = model_selection.cross_val_score(msgd1,X,y,cv=cvEval,scoring='roc_auc')

#valeurs AUC
print(auc)

#AUC moyenne
print(auc.mean())

#fonction pour évaluer un classifieur
def evaluation(classifier,cv,X,y):
    #learning process
    classifier.fit(X,y)
    #nombre d'itérations
    print("ITERATIONS = ",classifier.n_iter_)
    #get the probabilities
    p = classifier.predict_proba(X)[:,1]
    #loss score
    ls = metrics.log_loss(y,p,normalize=True,eps=1e-15)
    print("LOSS = ",ls)
    #auc
    auc = model_selection.cross_val_score(classifier,X,y,cv=cv,scoring='roc_auc')
    print("AUC = ", auc.mean())
    #
    return classifier.n_iter_, ls, auc.mean()
    

#modifier la règle d'arrêt
msgd2 = SGDClassifier(loss="log",penalty="none",learning_rate="constant",eta0=0.1, max_iter=None,tol=1e-5)

#evaluation
evaluation(msgd2,cvEval,X,y)

#décroissance de eta
msgd3 = SGDClassifier(loss="log",penalty="none",learning_rate="invscaling",eta0=0.1, max_iter=None,tol=1e-5)

#evaluation
evaluation(msgd3,cvEval,X,y)

#paramétrage par défaut
msgd4 = SGDClassifier(loss="log",penalty="none")

#evaluation
evaluation(msgd4,cvEval,X,y)

#liste pour construction des courbes ROC
lstRoc = []

#courbe ROC en validation croisée
for idTrain,idTest in cvEval.split(X,y):
    #processus d'apprentissage
    msgd2.fit(X[idTrain,:],y[idTrain])
    #probabilités d'appartenir à la classe 1
    p = msgd2.predict_proba(X[idTest,:])[:,1]
    #valeurs pour la courbe roc
    fpr,tpr,thresholds = metrics.roc_curve(y[idTest],p)
    #ajouter
    lstRoc.append({'fpr':fpr,'tpr':tpr})
    
#nombre de couples de valeurs dispo
print(len(lstRoc))

#graphiques
import matplotlib.pyplot as plt

#les 3 courbes ROC
for v in lstRoc:
    plt.plot(v['fpr'],v['tpr'],lw=1)
    
#diagonale
plt.plot([0,1],[0,1],linestyle='--',lw=1,color='black')

#légendes
plt.xlabel('False Positive Rate')
plt.ylabel('True Positibve Rate')

#titre
plt.title('[CV] ROC curve for each test fold')
plt.show()

#coordonnées en abscisse courbe n°0
print("FPR(0)",lstRoc[0]['fpr'])

#coordonnées en abscisse courbe n°1
print("FPR(1)",lstRoc[1]['fpr'])

#coordonnées en abscisse courbe n°2
print("FPR(2)",lstRoc[2]['fpr'])

#suite de 20 points équidistants
mfpr = numpy.linspace(0,1,20)
print(mfpr)

#les 3 suites de valeurs de tpr pour ces valeurs
import scipy

#liste vide
lst_tpr = []

#pour chaque courbe
for curve in lstRoc:
    #estimer les valeurs de tpr pour les coordonnées de mfpr
    estimated_tpr = scipy.interp(mfpr,curve['fpr'],curve['tpr'])
    #première valeur en ordonnée = 0
    estimated_tpr[0] = 0.0
    #ajout dans la liste
    lst_tpr.append(estimated_tpr)
    
#transformation en matrice
mlst_tpr = numpy.array(lst_tpr)
print(mlst_tpr)

#vérification des dimensions
print(mlst_tpr.shape)

#ordonnée moyenne
mtpr = numpy.mean(mlst_tpr,axis=0)
print(mtpr)

#plotting
plt.plot(mfpr,mtpr,lw=2,color='y')

#diagonale
plt.plot([0,1],[0,1],linestyle='--',lw=1,color='k')

#légendes
plt.xlabel('False Positive Rate')
plt.ylabel('True Positibve Rate')

#titre
plt.title('[CV] Average ROC curve')
plt.show()

#AUC de la courbe synthétique
print(metrics.auc(mfpr,mtpr))
